package java.util;

import java.lang.reflect.Array;

public abstract class AbstractCollection<E> implements Collection<E>
{
    private static final int MAX_ARRAY_SIZE = 2147483639;
    
    protected AbstractCollection() {
    }
    
    @Override
    public abstract Iterator<E> iterator();
    
    @Override
    public abstract int size();
    
    @Override
    public boolean isEmpty() {
        return this.size() == 0;
    }
    
    @Override
    public boolean contains(Object o) {
        Iterator<E> iterator = this.iterator();
        if (o == null) {
            while (iterator.hasNext()) {
                if (iterator.next() != null) {
                    continue;
                }
                return true;
            }
        }
        else {
            while (iterator.hasNext()) {
                if (!o.equals(iterator.next())) {
                    continue;
                }
                return true;
            }
        }
        return false;
    }
    
    @Override
    public Object[] toArray() {
        Object[] array = new Object[this.size()];
        Iterator<E> iterator = this.iterator();
        int i = 0;
        while (i < array.length) {
            if (!iterator.hasNext()) {
                return Arrays.copyOf(array, i);
            }
            array[i] = iterator.next();
            ++i;
        }
        return iterator.hasNext() ? finishToArray(array, iterator) : array;
    }
    
    @Override
    public <T> T[] toArray(T[] array) {
        int size = this.size();
        Object[] array2 = (array.length >= size) ? array : ((Object[])(Object[])Array.newInstance(array.getClass().getComponentType(), size));
        Iterator<E> iterator = this.iterator();
        int i = 0;
        while (i < array2.length) {
            if (!iterator.hasNext()) {
                if (array == array2) {
                    array2[i] = null;
                }
                else {
                    if (array.length < i) {
                        return Arrays.copyOf(array2, i);
                    }
                    System.arraycopy(array2, 0, array, 0, i);
                    if (array.length > i) {
                        array[i] = null;
                    }
                }
                return array;
            }
            else {
                array2[i] = iterator.next();
                ++i;
            }
        }
        return iterator.hasNext() ? finishToArray(array2, iterator) : array2;
    }
    
    private static <T> T[] finishToArray(T[] copy, Iterator<?> iterator) {
        int length;
        for (length = copy.length; iterator.hasNext(); copy[length++] = iterator.next()) {
            int length2 = copy.length;
            if (length == length2) {
                int hugeCapacity = length2 + (length2 >> 1) + 1;
                if (hugeCapacity - 2147483639 > 0) {
                    hugeCapacity = hugeCapacity(length2 + 1);
                }
                copy = Arrays.copyOf(copy, hugeCapacity);
            }
        }
        return (length == copy.length) ? copy : Arrays.copyOf(copy, length);
    }
    
    private static int hugeCapacity(int n) {
        if (n < 0) {
            throw new OutOfMemoryError("Required array size too large");
        }
        return (n > 2147483639) ? Integer.MAX_VALUE : 2147483639;
    }
    
    @Override
    public boolean add(E e) {
        throw new UnsupportedOperationException();
    }
    
    @Override
    public boolean remove(Object o) {
        Iterator<E> iterator = this.iterator();
        if (o == null) {
            while (iterator.hasNext()) {
                if (iterator.next() != null) {
                    continue;
                }
                iterator.remove();
                return true;
            }
        }
        else {
            while (iterator.hasNext()) {
                if (!o.equals(iterator.next())) {
                    continue;
                }
                iterator.remove();
                return true;
            }
        }
        return false;
    }
    
    @Override
    public boolean containsAll(Collection<?> collection) {
        Iterator<Object> iterator = collection.iterator();
        while (iterator.hasNext()) {
            if (!this.contains(iterator.next())) {
                return false;
            }
            continue;
        }
        return true;
    }
    
    @Override
    public boolean addAll(Collection<? extends E> collection) {
        boolean n = false;
        Iterator<Object> iterator = collection.iterator();
        while (iterator.hasNext()) {
            if (this.add(iterator.next())) {
                n = true;
            }
            else {
                continue;
            }
        }
        return n;
    }
    
    @Override
    public boolean removeAll(Collection<?> collection) {
        Objects.requireNonNull(collection);
        boolean n = false;
        Iterator<E> iterator = this.iterator();
        while (iterator.hasNext()) {
            if (!collection.contains(iterator.next())) {
                continue;
            }
            iterator.remove();
            n = true;
        }
        return n;
    }
    
    @Override
    public boolean retainAll(Collection<?> collection) {
        Objects.requireNonNull(collection);
        boolean n = false;
        Iterator<E> iterator = this.iterator();
        while (iterator.hasNext()) {
            if (collection.contains(iterator.next())) {
                continue;
            }
            iterator.remove();
            n = true;
        }
        return n;
    }
    
    @Override
    public void clear() {
        Iterator<E> iterator = this.iterator();
        while (iterator.hasNext()) {
            iterator.next();
            iterator.remove();
        }
    }
    
    @Override
    public String toString() {
        Iterator<E> iterator = this.iterator();
        if (!iterator.hasNext()) {
            return "[]";
        }
        StringBuilder sb = new StringBuilder();
        sb.append('[');
        while (true) {
            Object next = iterator.next();
            sb.append((next == this) ? "(this Collection)" : next);
            if (!iterator.hasNext()) {
                break;
            }
            sb.append(',').append(' ');
        }
        return sb.append(']').toString();
    }
}
